import argparse
from importlib import resources
from btweak import __version__
from btweak.helpers.fixthings import fix_berserkarch_gpg_pacman, fix_db_lck
from btweak.helpers.toolhandler import (
    print_groups,
    print_specific_group_by_index,
    install_group,
)  # noqa
from btweak.helpers.fileparser import ToolGroupParser, ContainersGroupParser
from btweak.helpers.dockerhandler import ContainerDisplay


def parse_args():
    parser = argparse.ArgumentParser(
        prog="btweak", description="Berserk Arch Tweak Tool"
    )

    parser.add_argument(
        "-v", "--version", action="version", version=f"%(prog)s {__version__}"
    )

    subcmd = parser.add_subparsers(dest="command", required=True)

    # fix things in berserkarch
    fix_subcmd = subcmd.add_parser("fix", help="Fix things in berserkarch")
    fix_subcmd.add_argument(
        "-g", "--gpg", action="store_true", help="Fix pacman gpg key issue"
    )
    fix_subcmd.add_argument(
        "--db-lck",
        action="store_true",
        help="Fix pacman -- unable to lock database error",
    )

    # tools group
    tools_subcmd = subcmd.add_parser(
        "tools", help="List, install and remove tools"
    )  # noqa
    tools_subcmd.add_argument(
        "-l",
        "--list",
        action="store_true",
        help="List all tools categories and profiles",
    )
    tools_subcmd.add_argument(
        "-g", "--group", type=int, help="List info about a specific group"
    )
    tools_subcmd.add_argument(
        "-i", "--install", type=int, help="Install a specific group"
    )

    # docker containers for systems and tools
    docker_subcmd = subcmd.add_parser(
        "docker", help="List and manage docker images for systems and tools"
    )
    docker_subcmd.add_argument(
        "-l",
        "--list",
        action="store_true",
        help="List all available options",
    )
    docker_subcmd.add_argument(
        "-g", "--group", type=int, help="List info about a specific group"
    )
    docker_subcmd.add_argument(
        "-c",
        "--category",
        type=int,
        help="List containers in a specific category. Must be used with the --group (-g) flag.",  # noqa
    )
    docker_subcmd.add_argument(
        "-s", "--search", type=str, help="Search for available containers"
    )

    docker_subcmd.add_argument(
        "-r", "--run", type=str, help="Run any available containers"
    )
    docker_subcmd.add_argument(
        "-t",
        "--terminal",
        action="store_true",
        help="Run container in separate terminal. Must be used with --run (-r) flag.",  # noqa
    )
    docker_subcmd.add_argument(
        "-f",
        "--flags",
        type=str,
        help="Additional flags to pass to docker run command. Must be used with --run (-r) flag.",  # noqa
    )
    docker_subcmd.add_argument(
        "-C",
        "--cleanup",
        action="store_true",
        help="Complete cleanup of things related to docker. BE CAREFUL - If you have any existing thing with docker, it all will be gone...",  # noqa
    )

    return parser, parser.parse_args()


def main():
    parser, args = parse_args()

    match args.command:
        case "fix":
            if args.gpg:
                fix_berserkarch_gpg_pacman()
            elif args.db_lck:
                fix_db_lck()
            else:
                parser.parse_args(["fix", "--help"])
        case "tools":
            toolsp = ToolGroupParser(
                resources.files("btweak.data").joinpath("tools.yaml")
            )
            groups = toolsp.parse()
            if args.list:
                print_groups(groups)
            elif args.group:
                print_specific_group_by_index(args.group, toolsp)
            elif args.install:
                install_group(args.install, toolsp)
            else:
                parser.parse_args(["tools", "--help"])

        case "docker":
            dockerp = ContainersGroupParser(
                resources.files("btweak.data").joinpath("docker.yaml")
            )
            dockerp.parse()
            docker = ContainerDisplay(dockerp)

            if args.group and args.category:
                docker.show_category(args.group, args.category)
            elif args.group:
                docker.show_group(args.group)
            elif args.category:
                print(
                    "Error: The --category (-c) flag must be used with the --group (-g) flag."  # noqa
                )
                parser.parse_args(["docker", "--help"])
            elif args.list:
                docker.show_all_groups()
            elif args.search:
                docker.search(args.search)
            elif args.run:
                if args.terminal:
                    docker.run(args.run, True, args.flags)
                else:
                    docker.run(args.run, False, args.flags)
            elif args.terminal:
                print(
                    "Error: The --terminal (-t) flag must be used with the --run (-r) flag."  # noqa
                )
                parser.parse_args(["docker", "--help"])
            elif args.flags:
                print(
                    "Error: The --flags (-f) flag must be used with the --run (-r) flag."  # noqa
                )
                parser.parse_args(["docker", "--help"])
            elif args.cleanup:
                docker.cleanup()
            else:
                parser.parse_args(["docker", "--help"])
        case _:
            parser.print_help()


if __name__ == "__main__":
    main()



from rich.console import Console
from rich.tree import Tree
from btweak.helpers.cmdhandler import (
    run_system_commands,
    get_cmd_data,
    remove_dir,
)  # noqa
from rich import print
import os
import shlex


class ContainerDisplay:
    def __init__(self, parser, console=None):
        self.parser = parser
        self.console = console or Console()

    def show_all_groups(self):
        main_tree = Tree("[bold blue]═══ Container Groups ═══[/]")

        for idx, group in enumerate(self.parser.container_groups, start=1):
            group_branch = main_tree.add(f"[bold cyan]{idx}. {group.name}[/]")
            group_branch.add(f"[dim italic]{group.description}[/]")

            if hasattr(group, "categories") and group.categories:
                total_containers = sum(
                    len(cat.containers) for cat in group.categories
                )  # noqa
                group_branch.add(
                    f"[yellow]{len(group.categories)} categories, {total_containers} total containers[/]"  # noqa
                )

                categories_branch = group_branch.add(
                    "[bold magenta]Categories:[/]"
                )  # noqa
                for cat_idx, category in enumerate(group.categories, start=1):
                    cat_branch = categories_branch.add(
                        f"[magenta]{cat_idx}. {category.name}[/] [dim]({len(category.containers)} containers)[/]"  # noqa
                    )
                    cat_branch.add(f"[dim italic]{category.description}[/]")

            elif hasattr(group, "containers") and group.containers:
                containers_branch = group_branch.add(
                    f"[yellow]{len(group.containers)} containers available[/]"
                )
                for container in group.containers:
                    containers_branch.add(f"[green]▸ {container.name}[/]")

        self._print_tree(main_tree)

    def show_group(self, index: int):
        group = self._get_group_or_error(index)
        if group is None:
            return

        main_tree = Tree(f"[bold blue]{group.name}[/]")
        main_tree.add(f"[dim italic]{group.description}[/]")

        if hasattr(group, "categories") and group.categories:
            self._add_categorized_containers(main_tree, group)
        elif hasattr(group, "containers") and group.containers:
            self._add_flat_containers(main_tree, group)

        self._print_tree(main_tree)

    def show_category(self, group_index: int, category_index: int):
        group = self._get_group_or_error(group_index)
        if group is None:
            return

        if not (hasattr(group, "categories") and group.categories):
            self._show_error(
                f"The selected group '{group.name}' does not contain any categories."  # noqa
            )
            return

        category = self._get_category_or_error(group, category_index)
        if category is None:
            return

        main_tree = Tree(
            f"[bold magenta]Category: {category.name}[/] [dim]from Group:[/] [bold blue]{group.name}[/]"  # noqa
        )
        main_tree.add(f"[dim italic]{category.description}[/]")
        main_tree.add(
            f"[yellow]Listing {len(category.containers)} container(s)[/]"
        )  # noqa

        for idx, container in enumerate(category.containers, start=1):
            self._add_container_details(main_tree, container, f"{idx}. ")

        self._print_tree(main_tree)

    def search(self, search_term: str):
        results = self.parser.search_container(search_term)

        if not results:
            tree = Tree(f"[bold yellow]Search: '{search_term}'[/]")
            tree.add("[dim]No results found[/]")
            self._print_tree(tree)
            return

        tree = Tree(f"[bold cyan]Search Results for '{search_term}'[/]")
        tree.add(f"[yellow]Found {len(results)} container(s)[/]")

        for result in results:
            container = result[-1]
            location = self._format_result_location(result)

            result_branch = tree.add(f"[green]{container.name}[/]")
            result_branch.add(location)
            self._add_container_details(result_branch, container, prefix="")

        self._print_tree(tree)

    def run(
        self, search_term: str, terminal: bool = False, additional_flags=None
    ):  # noqa
        results = self.parser.search_container(search_term)

        if not results:
            tree = Tree(f"[bold yellow]Search: '{search_term}'[/]")
            tree.add("[dim]No results found[/]")
            self._print_tree(tree)
            return

        if len(results) == 1:
            container = results[0][-1]
            self._execute_container(container, terminal, additional_flags)
            return

        tree = Tree(f"[bold cyan]Multiple Results for '{search_term}'[/]")
        tree.add(f"[yellow]Found {len(results)} container(s)[/]")

        for result in results:
            container = result[-1]
            location = self._format_result_location(result)

            result_branch = tree.add(f"[green]{container.name}[/]")
            result_branch.add(location)
            self._add_container_details(result_branch, container, prefix="")

        self._print_tree(tree)

    def cleanup(self):
        print("([b yellow]*[/]) Starting complete docker obliteration...\n")
        self._get_output_run_cmd(
            [["docker", "ps", "-q"], "docker stop "],
            "No Running containers found...",  # noqa
        )
        self._get_output_run_cmd(
            [
                [
                    "docker",
                    "images",
                    "-q",
                ],
                "docker rmi -f ",
            ],
            "No Images were found...",
        )
        run_system_commands("docker system prune -f")
        remove_dir("~/btweak/docker")
        remove_dir("~/btweak/containers")
        print("\n([b green]*[/]) Done...")

    def _get_group_or_error(self, index: int):
        group = self.parser.get_group_by_index(index)
        if group is None:
            self._show_error(
                f"Invalid index: {index}",
                f"Available indices: 1-{len(self.parser.container_groups)}",
            )
        return group

    def _get_category_or_error(self, group, category_index: int):
        try:
            return group.categories[category_index - 1]
        except IndexError:
            self._show_error(
                f"Category with index '{category_index}' not found in group '{group.name}'.",  # noqa
                f"Available indices for this group: 1 to {len(group.categories)}",  # noqa
            )
            return None

    def _add_categorized_containers(self, tree, group):
        total_containers = sum(len(cat.containers) for cat in group.categories)
        tree.add(
            f"[yellow]Total: {len(group.categories)} categories, {total_containers} containers[/]"  # noqa
        )

        for cat_idx, category in enumerate(group.categories, start=1):
            category_branch = tree.add(
                f"[bold magenta]{cat_idx}. {category.name}[/]"
            )  # noqa
            category_branch.add(f"[dim italic]{category.description}[/]")

            for idx, container in enumerate(category.containers, start=1):
                self._add_container_details(
                    category_branch, container, f"{cat_idx}.{idx}. "
                )

    def _add_flat_containers(self, tree, group):
        tree.add(f"[yellow]Total: {len(group.containers)} containers[/]")

        for idx, container in enumerate(group.containers, start=1):
            self._add_container_details(tree, container, f"{idx}. ")

    def _add_container_details(self, parent_branch, container, prefix=""):
        container_branch = parent_branch.add(
            f"[bold green]{prefix}{container.name}[/]"
        )  # noqa
        container_branch.add(f"[white]{container.description}[/]")

        commands_branch = container_branch.add("[bold cyan]Commands:[/]")
        commands_branch.add(f"[blue]Pull:[/] {container.command}")
        commands_branch.add(f"[yellow]Run:[/] {container.run}")

    def _format_result_location(self, result):
        if len(result) == 3:
            group_name, category_name, _ = result
            return f"[dim]Group: {group_name} → Category: {category_name}[/]"
        else:
            group_name, _ = result
            return f"[dim]Group: {group_name}[/]"

    def _execute_container(self, container, terminal, additional_flags=None):
        mod_cmd = container.run.replace("~", os.path.expanduser("~"))
        if additional_flags:
            mod_cmd = mod_cmd.replace(
                "docker run", f"docker run {additional_flags}", 1
            )  # noqa
        print(mod_cmd)
        if container.runtime_comments:
            print("\n=== [b green]Runtime Information[/] ===")
            for i in container.runtime_comments:
                print(i)
            print("=== [b red]Runtime Information[/] ===\n")
        if terminal:
            run_system_commands([f"kitty --hold tmux new-session {mod_cmd}"])  # noqa
        else:
            # run_system_commands([f"{container.run}"], interactive=True)
            # mod_cmd = container.run.replace("~", os.path.expanduser("~"))
            part_cmd = shlex.split(mod_cmd)
            os.execvp(part_cmd[0], part_cmd)

    def _show_error(self, *messages):
        error_tree = Tree("[bold red]✗ Error[/]")
        for msg in messages:
            error_tree.add(msg)
        self._print_tree(error_tree)

    def _print_tree(self, tree):
        self.console.print()
        self.console.print(tree)
        self.console.print()

    def _get_output_run_cmd(self, cmds: list[list | str], msg):
        result = get_cmd_data(cmds[0])
        if result:
            run_system_commands(cmds[1] + " ".join(result))
        else:
            print(msg)



from btweak.helpers.cmdhandler import run_system_commands
from os.path import exists
import subprocess
import sys
from rich import print


def fix_berserkarch_gpg_pacman():
    fix_db_lck()
    run_system_commands(
        [
            "sudo pacman-key --init",
            "sudo pacman-key --populate",
            "curl -s https://thehackersbrain.dev/pubkey.asc | gpg --import",
            "gpg --export B024DCEFADEF4328B5E3A848E7E0F2B78484DACF | sudo pacman-key --add -",  # noqa
            "sudo pacman-key --lsign-key B024DCEFADEF4328B5E3A848E7E0F2B78484DACF",  # noqa
            "sudo pacman -Syy --noconfirm",
        ]
    )


def fix_db_lck():
    fname = "/var/lib/pacman/db.lck"
    if exists(fname):
        result = subprocess.run(
            ["pgrep", "-x", "pacman"], capture_output=True, text=True
        )
        if result.stdout.strip():
            print("(x) Pacman is currently running! Aborting unlock.")
            sys.exit(1)
        else:
            run_system_commands("sudo rm -rf /var/lib/pacman/db.lck")
    else:
        print("(✓) No lock file found. Pacman DB is clean.")



from rich.console import Console
from rich.tree import Tree
from btweak.helpers.cmdhandler import run_system_commands
from btweak.helpers.fixthings import fix_db_lck

console = Console()


def print_groups(tool_groups):
    tree = Tree("[=== [b u blue]All Tool Groups[/] ===]")
    for ind, i in enumerate(tool_groups, start=1):
        group = tree.add(f"{ind}. [b cyan]{i.name}[/]")
        group.add(f"[dim]{i.description}[/]")
        group.add(f"[yellow]{len(i.packages)} packages[/]")
    console.print(tree)
    print()


def print_specific_group_by_index(index: int, parser):
    group = parser.get_group_by_index(index)
    if group is None:
        console.print(f"[red]Error: Invalid index {index}[/]")
        return

    tree = Tree(f"[b u cyan]{group.name}[/]")
    tree.add(f"[dim]{group.description}[/]")

    packages_branch = tree.add(f"[yellow]{len(group.packages)} packages[/]")
    for i in group.packages:
        packages_branch.add(f"[green]{i.name}[/]").add(
            f"[dim]{i.description}[/]"
        )  # noqa

    console.print(tree)
    print()


def install_group(index: int, parser):
    pkgs = " ".join(i.name for i in parser.get_packages_by_index(index))
    fix_db_lck()

    run_system_commands(
        [
            "sudo pacman -Syy --noconfirm yay",
            "yay -Syy --noconfirm {}".format(pkgs),
        ]  # noqa
    )



import subprocess
from typing import Union, List, Dict
from rich import print
from pathlib import Path


def run_system_commands(
    commands: Union[str, List[str]],
    shell: bool = True,
    text: bool = True,
    check: bool = False,
    interactive: bool = False,
) -> Union[Dict, List[Dict]]:

    def execute_single_command(cmd: str) -> Dict:
        if interactive:
            subprocess.run(cmd, shell=shell)
        else:
            cmd_args = cmd if shell else cmd.split()

            print("[b]>>> {}[/]".format(cmd))
            process = subprocess.Popen(
                cmd_args if isinstance(cmd_args, str) else cmd_args,
                shell=shell,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=text,
                bufsize=1,
            )

            output = []
            for line in process.stdout:
                print(line, end="")
                output.append(line)

            process.wait()

    if isinstance(commands, str):
        execute_single_command(commands)
    elif isinstance(commands, list):
        [execute_single_command(cmd) for cmd in commands]
    else:
        raise TypeError("commands must be a string or list of strings")


def get_cmd_data(cmds: List[str]) -> List:
    result = subprocess.run(cmds, capture_output=True, text=True)
    output = result.stdout.strip().splitlines()

    return output


def remove_dir(path: str) -> bool:
    p = Path(path).expanduser()
    if p.exists():
        run_system_commands(f"sudo rm -rf {path}")
    else:
        print(f"[info] Directory does not exists: {p}")
        return False
    return True



import yaml
from typing import List, Dict, Optional
from dataclasses import dataclass, field


@dataclass
class Package:
    name: str
    description: str


@dataclass
class ToolGroup:
    name: str
    description: str
    packages: List[Package] = field(default_factory=list)


@dataclass
class Container:
    name: str
    description: str
    command: str
    run: str
    runtime_comments: Optional[List[str]] = None


@dataclass
class Category:
    name: str
    description: str
    containers: List[Container] = field(default_factory=list)


@dataclass
class ContainersGroup:
    name: str
    description: str
    containers: List[Container] = field(default_factory=list)
    categories: List[Category] = field(default_factory=list)


class ToolGroupParser:
    def __init__(self, yaml_file: str):
        self.yaml_file = yaml_file
        self.tool_groups: List[ToolGroup] = []

    def parse(self) -> List[ToolGroup]:
        with open(self.yaml_file, "r") as f:
            data = yaml.safe_load(f)

        for i in data:
            packages = []
            for pkg_data in i.get("packages", []):
                pkg = Package(
                    name=pkg_data["name"],
                    description=pkg_data["description"],
                )
                packages.append(pkg)

            tool_group = ToolGroup(
                name=i["name"],
                description=i["description"],
                packages=packages,
            )
            self.tool_groups.append(tool_group)

        return self.tool_groups

    def get_packages_by_index(self, index: int) -> Optional[List[Package]]:
        try:
            return self.tool_groups[index - 1].packages
        except IndexError:
            return None

    def get_group_by_index(self, index: int) -> Optional[ToolGroup]:
        try:
            return self.tool_groups[index - 1]
        except IndexError:
            return None

    def search_package(self, search_term: str) -> List[tuple]:
        results = []
        for group in self.tool_groups:
            for pkg in group.packages:
                if search_term.lower() in pkg.name.lower():
                    results.append((group.name, pkg))
        return results

    def get_all_packages(self) -> Dict[str, List[Package]]:
        return {group.name: group.packages for group in self.tool_groups}


class ContainersGroupParser:
    def __init__(self, yaml_file: str):
        self.yaml_file = yaml_file
        self.container_groups: List[ContainersGroup] = []

    def parse(self) -> List[ContainersGroup]:
        with open(self.yaml_file, "r") as f:
            data = yaml.safe_load(f)

        for i in data:
            container_group = ContainersGroup(
                name=i["name"],
                description=i["description"],
            )

            if "categories" in i:
                for cat_data in i["categories"]:
                    containers = []
                    for container_data in cat_data.get("containers", []):
                        container = Container(
                            name=container_data["name"],
                            description=container_data["description"],
                            command=container_data["command"],
                            run=container_data["run"],
                            runtime_comments=container_data.get(
                                "runtime_comments"
                            ),  # noqa
                        )
                        containers.append(container)

                    category = Category(
                        name=cat_data["name"],
                        description=cat_data["description"],
                        containers=containers,
                    )
                    container_group.categories.append(category)

            elif "containers" in i:
                for container_data in i["containers"]:
                    container = Container(
                        name=container_data["name"],
                        description=container_data["description"],
                        command=container_data["command"],
                        run=container_data["run"],
                        runtime_comments=container_data.get("runtime_comments"),  # noqa
                    )
                    container_group.containers.append(container)

            self.container_groups.append(container_group)

        return self.container_groups

    def get_containers_by_index(self, index: int) -> Optional[List[Container]]:
        try:
            group = self.container_groups[index - 1]
            if group.categories:
                all_containers = []
                for category in group.categories:
                    all_containers.extend(category.containers)
                return all_containers
            return group.containers
        except IndexError:
            return None

    def get_group_by_index(self, index: int) -> Optional[ContainersGroup]:
        try:
            return self.container_groups[index - 1]
        except IndexError:
            return None

    def search_container(self, search_term: str) -> List[tuple]:
        results = []
        for group in self.container_groups:
            for container in group.containers:
                if search_term.lower() in container.name.lower():
                    results.append((group.name, container))

            for category in group.categories:
                for container in category.containers:
                    if search_term.lower() in container.name.lower():
                        results.append((group.name, category.name, container))

        return results

    def get_all_containers(self) -> Dict[str, List[Container]]:
        result = {}
        for group in self.container_groups:
            all_containers = []
            all_containers.extend(group.containers)
            for category in group.categories:
                all_containers.extend(category.containers)
            result[group.name] = all_containers
        return result
